/*-----------------------------------------------------------------------+
 | edu.tum.cs.ccts.model.base
 |                                                                       |
   $Id$            
 |                                                                       |
 | Copyright (c)  2004-2008 Technische Universitaet Muenchen             |
 |                                                                       |
 | Technische Universitaet Muenchen               #########  ##########  |
 | Institut fuer Informatik - Lehrstuhl IV           ##  ##  ##  ##  ##  |
 | Prof. Dr. Manfred Broy                            ##  ##  ##  ##  ##  |
 | Boltzmannstr. 3                                   ##  ##  ##  ##  ##  |
 | 85748 Garching bei Muenchen                       ##  ##  ##  ##  ##  |
 | Germany                                           ##  ######  ##  ##  |
 +-----------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.internal.views;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.ui.actions.CompoundContributionItem;
import org.fortiss.tooling.kernel.interfaces.IPrototypeProvider;
import org.fortiss.tooling.kernel.services.ICommandStackService;
import org.fortiss.tooling.kernel.services.ICompositorService;
import org.fortiss.tooling.kernel.services.IPrototypeService;
import org.fortiss.tooling.kernel.services.IPrototypeService.Prototype;
import org.fortiss.tooling.kernel.ui.services.IEditorService;
import org.fortiss.tooling.kernel.ui.services.IModelElementService;
import org.fortiss.tooling.kernel.ui.services.INavigatorService;
import org.fortiss.tooling.kernel.ui.util.EObjectSelectionUtils;

/**
 * The contributor for the dynamic new menu based on composition.
 * 
 * @author hoelzlf
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash: 14F19ABE1F8516F2D29342D5136D1FD6
 */
public class NewMenu extends CompoundContributionItem {

	/** The menu id. */
	public static final String MENU_ID = "org.fortiss.tooling.kernel.navigator.newmenu";

	/** List of available actions. */
	private final List<AddPrototypeAction> actions = new ArrayList<AddPrototypeAction>();

	/** Constructor. */
	public NewMenu() {
		for (IPrototypeProvider provider : IPrototypeService.INSTANCE
				.getPrototypeProviders()) {
			for (Prototype prototype : provider.getPrototypes()) {
				actions.add(new AddPrototypeAction(prototype));
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	protected IContributionItem[] getContributionItems() {
		// FIXME (FH): this assumes the navigator is providing the
		// selection, which might not be what we want!
		EObject selectedObject = EObjectSelectionUtils
				.getFirstElement(INavigatorService.INSTANCE
						.getCurrentSelection());

		List<IContributionItem> contributionItems = new ArrayList<IContributionItem>();
		if (selectedObject != null) {
			for (AddPrototypeAction apa : actions) {
				if (apa.prepare(selectedObject)) {
					contributionItems.add(new ActionContributionItem(apa));
				}
			}
		}
		return contributionItems
				.toArray(new IContributionItem[contributionItems.size()]);
	}

	/** Action for creating a new prototype. */
	private static class AddPrototypeAction extends Action {

		/** The prototype. */
		private final Prototype prototype;

		/** The container. */
		private EObject container;

		/** Constructor. */
		public AddPrototypeAction(Prototype prototype) {
			super(prototype.getName(), IModelElementService.INSTANCE
					.getModelElementHandler(prototype.getPrototype())
					.getIconImageDescriptor());
			this.prototype = prototype;
		}

		/** {@inheritDoc} */
		@Override
		public void run() {
			ICommandStackService.INSTANCE.runAsCommand(container,
					new Runnable() {

						@Override
						public void run() {
							EObject newObject = prototype.getPrototypeCopy();
							if (ICompositorService.INSTANCE.compose(container,
									newObject, null)) {
								IEditorService.INSTANCE.openInEditor(newObject);
							}
						}
					});
		}

		/**
		 * Stores the given container and returns whether the underlying
		 * prototype can be added to it.
		 */
		public boolean prepare(EObject container) {
			this.container = container;
			return ICompositorService.INSTANCE.canCompose(container,
					prototype.getPrototype(), null);
		}

	}
}
